
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% DescentTest.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [point,par,info]=descentTest(fun,point,step,par,tune,info);
%
% try to significantly improve function value by Delta
%
% inputs: fun,point,step,par,tune

% outputs: point,good,lam,info

function [point,par,step,info] = MLS(fun,point,step,par,tune,info)
point.xm=point.X(:,point.b); point.fm = point.F(point.b);
point.xr = point.xm; point.finit=point.fm; 
stateTest = ''; step.r = 0;  step.q = zeros(point.n,1); par.good=0;
t=0;par.state=-1; prt=(info.prt>=1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%% main loop %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
while t<par.T
  switch par.state
  case -1 % new direction
    t=t+1;  nE=0;  par.alphaE = par.A(t); 
    par.t=t;
    if t<=tune.C
      par.dir       = 1;   % pick coordinate direction
      if prt
        stepType      = 'Co'; % coordinate direction
        info.nstep.nCo = info.nstep.nCo+1; 
      end
    elseif t== tune.C+1 % pick quasi Newton direction
          par.dir = 2;   % pick quasi Newton direction
          if prt
             stepType      = 'Q'; % quasi Newton direction
             info.nstep.nQ = info.nstep.nQ+1; 
          end
    elseif t<=tune.C +tune.S+1 & point.m>=3
       par.dir       = 3;   % pick random subspace direction
      if prt
        stepType      = 'S'; % random subspace direction
        info.nstep.nS = info.nstep.nS+1; 
      end  
    elseif t<par.T 
          par.dir   = 4;   % pick random direction
          if prt
          stepType  = 'R'; % Random direction
          info.nstep.nR = info.nstep.nR+1;
          end
    else
      if tune.cum==2 & step.r>=step.Delta % second cum step
      elseif tune.cum==1, step.q=point.x-point.xinit; % first cum step
      end
      if tune.cum>0 & all(step.q==0)
          par.dir=3; % pick random subspace step
         if prt 
           stepType  = 'S'; % subspace direction
           info.nstep.nS= info.nstep.nS+1; 
         end
      else
        par.dir = 5; % pick cumulative step
        if tune.cum==1, par.state = 1; point.fl = point.finit; end 
                       % use initial value
         if  prt             
            stepType  = 'Cu'; % cumulstive direction
            info.nstep.nCu = info.nstep.nCu+1; 
         end
      end
    end
    
    [step,par] = direction(point,step,par,tune);
    
  case 0  % extrapolate 
      
    nE=nE+1;
    point.fext = point.fr; 
    if nE==1
       point.fl = point.fm;
    else
       par.alphaE = tune.gammaE*par.alphaE; 
    end
    
    if prt
      stepType  = 'E'; % extrapolate
      info.nstep.nE = info.nstep.nE+1;
    end
    
  case 1 % opposite direction
     point.fl   = point.fr;
     par.alphaE = par.A(t);  
     step.p     = -step.p;
     if prt
        stepType  = 'O'; % opposite direction
        info.nstep.nO = info.nstep.nO+1;
     end
     nE = 0;
  end
  
    if prt, stateTest=[stateTest,stepType], end;% update state
  
     if par.dir==1
        point.xr(t) = point.xm(t)+par.alphaE*step.p;
     else
        point.xr = point.xm+par.alphaE*step.p;
     end
     
      point.fr = fun(point.xr);
      if isnan(point.fr), point.fr=inf; end;
      info.nf   = info.nf+1; 
      point.fr  = max(-1e50,min(point.fr,1e50));
      
      if par.dir==1 & par.state==-1
         point.grad(t,:)  = (point.fr-point.finit)/(par.alphaE);
      end
      
      % check stopping test
      sec=(cputime-info.initTime);
      info.done=(sec>info.secmax)|(info.nf>=info.nfmax);
      info.done=info.done|(point.fr<=info.ftarget);
      info.sec = sec;

     df = point.fm-point.fr;
     
     if (nE==1)
        par.lam = max(par.lam,abs(point.fl+point.fr-2*point.fm)...
                  /(step.dp)^2);
     end
     
    if tune.cum ==2, [step] = updatecum(point,step,tune); end
    
   
    ext = (df>step.Delta*par.alphaE);
    
    if ext, par.state=0; par.good=1; continue;
    else
        opp = (par.state<0);
        if opp, par.state=1; continue;end
        if par.state==0 
          %%%%%%%%%%%%%%%%%%%%%%%%%%%%
          % the end of extrapolation %
          %%%%%%%%%%%%%%%%%%%%%%%%%%%% 
           par.alphaE    = par.alphaE/tune.gammaE;
           par.A(t) = par.alphaE;
           
           if par.dir==1
              point.xm(t) = point.xm(t) + par.alphaE*step.p;
              point.xr(t) = point.xm(t);
           else
              point.xm = point.xm+ par.alphaE*step.p;
           end
           
           point.fm = point.fext; 
           %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        else
            if par.dir==1 
               point.xr(t) = point.xm(t);
            end
            par.A(t) = par.A(t)/tune.gammaE;
        end
       
        if prt
          stepType = 'U';
          stateTest=[stateTest,stepType];
        end
        par.state = -1;
    end
    
   
end  % of while T>0

info.laststate=stateTest;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
