
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% direction.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [step,par] = direction(point,step,par,tune,info)
% generate coordinate, quasi-Newton, subspace, random and
% cumulative directions
%
% for details of input and output structures see VSBBO.m
%
function [step,par] = direction(point,step,par,tune)
switch par.dir
            
   case 1 % coordinate direction
        step.p  = 1;  
        scale   = 0;  
   case 2 % quasi Newton direction
        if point.col==0
           step.p = -point.grad/norm(point.grad); 
        else
           step     = lbfgs(point,step);
           sigma    = point.grad'*step.p;
          if sigma>0,
             % move away from maximizer or saddle point
            step.p=-step.p;sigma=-sigma;
          end;
          sigma1  =  point.grad'* point.grad;
          sigma2  = step.p'*step.p;
          sigma12 = sigma1*sigma2;
          c       = sigma/sqrt(sigma12);
          if c<-tune.gammaangle,
          else
             w=(sigma12*max(tune.gammaw,1-c^2))/(1-tune.gammaangle^2);
            t=(sigma+tune.gammaangle*sqrt(w))/sigma1;
            if w>0 & isfinite(t), 
              step.p=step.p-t* point.grad;
            else 
              step.p  = - point.grad;
            end;
         end; 

        end
        scale   = 0;            
        
   case 3 % random subspace direction
        alpha = rand(point.m,1)-0.5;
        alpha=alpha/norm(alpha);
        for i=1:point.m
            dX(:,i) = alpha(i)*(point.X(:,i)-point.xm);
        end
        step.p=sum(dX')'; 
        scale = tune.scSub; 
  
   case 4 % random direction
        step.p = rand(point.n,1)-0.5;
        step.p(step.p==0) = 0.5; 
        scale = 1; 
   case 5 % cumulative direction
        step.p=step.q; scale = tune.scCum;
end

if scale % scale to ||p||=delta
     step.delta = max(step.deltamin,...
     min(sqrt(tune.gammadelta*step.Delta/par.lam),step.deltamax));
     step.p=step.s.*step.p*(step.delta/norm(step.p));  
     step.dp = step.delta;  
else
    step.dp = norm(step.p);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
